@@ -384,8 +384,8 @@ static size_t rtcp_app_init(RtpSession *session, uint8_t *buf, uint8_t subtype,
 	if (size<sizeof(rtcp_app_t)) return 0;
 	rtcp_common_header_init(&app->ch,session,RTCP_APP,subtype,size);
 	app->ssrc=htonl(session->snd.ssrc);
-	memset(app->name, 0, 4);
-	strncpy(app->name, name, sizeof(app->name) - 1);
+	memset(app->name, 0, sizeof(app->name));
+	memcpy(app->name, name, sizeof(app->name));
 	return sizeof(rtcp_app_t);
 }
 
